/****************************************************************************
**
** Copyright (C) 2015 The Qt Company Ltd.
** Contact: http://www.qt.io/licensing/
**
** This file is part of the QtCanvas3D module of the Qt Toolkit.
**
** $QT_BEGIN_LICENSE:LGPL3$
** Commercial License Usage
** Licensees holding valid commercial Qt licenses may use this file in
** accordance with the commercial license agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and The Qt Company. For licensing terms
** and conditions see http://www.qt.io/terms-conditions. For further
** information use the contact form at http://www.qt.io/contact-us.
**
** GNU Lesser General Public License Usage
** Alternatively, this file may be used under the terms of the GNU Lesser
** General Public License version 3 as published by the Free Software
** Foundation and appearing in the file LICENSE.LGPLv3 included in the
** packaging of this file. Please review the following information to
** ensure the GNU Lesser General Public License version 3 requirements
** will be met: https://www.gnu.org/licenses/lgpl.html.
**
** GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU
** General Public License version 2.0 or later as published by the Free
** Software Foundation and appearing in the file LICENSE.GPL included in
** the packaging of this file. Please review the following information to
** ensure the GNU General Public License version 2.0 requirements will be
** met: http://www.gnu.org/licenses/gpl-2.0.html.
**
** $QT_END_LICENSE$
**
****************************************************************************/

#include "framebuffer3d_p.h"

QT_BEGIN_NAMESPACE
QT_CANVAS3D_BEGIN_NAMESPACE

/*!
 * \qmltype Canvas3DFrameBuffer
 * \since QtCanvas3D 1.0
 * \inqmlmodule QtCanvas3D
 * \brief Contains an OpenGL framebuffer.
 *
 * An uncreatable QML type that contains an OpenGL framebuffer object. You can get it by calling the
 * \l{Context3D::createFramebuffer()}{Context3D.createFramebuffer()} method.
 */

/*!
 * \internal
 */
CanvasFrameBuffer::CanvasFrameBuffer(QObject *parent) :
    CanvasAbstractObject(parent),
    m_framebufferId(0),
    m_texture(0)
{
    initializeOpenGLFunctions();
    glGenFramebuffers(1, &m_framebufferId);
}

/*!
 * \internal
 */
CanvasFrameBuffer::~CanvasFrameBuffer()
{
    del();
}

/*!
 * \internal
 */
bool CanvasFrameBuffer::isAlive()
{
    return (m_framebufferId);
}

/*!
 * \internal
 */
void CanvasFrameBuffer::del()
{
    if (m_framebufferId) {
        glDeleteFramebuffers(1, &m_framebufferId);
        m_framebufferId = 0;
    }
}

/*!
 * \internal
 */
GLuint CanvasFrameBuffer::id()
{
    return m_framebufferId;
}

/*!
 * \internal
 */
void CanvasFrameBuffer::setTexture(CanvasTexture *texture)
{
    m_texture = texture;
}

/*!
 * \internal
 */
CanvasTexture *CanvasFrameBuffer::texture()
{
    return m_texture;
}

/*!
 * \internal
 */
QDebug operator<<(QDebug dbg, const CanvasFrameBuffer *buffer)
{
    if (buffer)
        dbg.nospace() << "Canvas3DFrameBuffer("<< buffer->name() <<", id:" << buffer->m_framebufferId << ")";
    else
        dbg.nospace() << "Canvas3DFrameBuffer("<< ((void*) buffer) <<")";
    return dbg.maybeSpace();
}

QT_CANVAS3D_END_NAMESPACE
QT_END_NAMESPACE
